#include "Multiplayer.h"
#include "RakServerInterface.h"
#include "RakClientInterface.h"
#include "RakNetworkFactory.h"
#include "BitStream.h"
#include "DataReplicator.h"
#include "Rand.h"
#include <stdlib.h> // atoi
#include <cstring> // strcpy

#ifdef _WIN32
#include <conio.h> // kbhit
#include <windows.h> // Sleep
#else
#include "../Unix/kbhit.h"
#include <unistd.h> // usleep
#endif

bool isServer;

class TestClass;
TestClass *testClass;
bool objectSynchronized;
bool memorySynchronized;
bool ownObject, ownMemory;


RakServerInterface *rakServer;
RakClientInterface *rakClient;

DataReplicator dataReplicator;

struct PositionAndVelocity
{
	float position, velocity;
	// Variables used for outputting for the test
	float lastPosition;
};

void SerializePositionAndVelocity(void *memory, unsigned byteLength, void *context, RakNet::BitStream *bitStream, void *lastSentValue, unsigned int currentTime, unsigned int lastSendTime, PlayerID playerId, bool *includeTimestamp)
{
	PositionAndVelocity *pav = (PositionAndVelocity*) memory;
	PositionAndVelocity *lastPav = (PositionAndVelocity*) lastSentValue;

	if (lastPav)
	{
		// WriteDelta means only write a single bit if the same value is the same as last time
		bitStream->WriteDelta(pav->position, lastPav->position);
		bitStream->WriteDelta(pav->velocity, lastPav->velocity);
	}
	else
	{
		// This version is when we don't know what the last value is
		bitStream->WriteDelta(pav->position);
		bitStream->WriteDelta(pav->velocity);
	}

	// Doing this allows the system to use accurate timestamps, but at the expense of 4 more bytes per packet
	*includeTimestamp=true;

	// Skip sending lastPosition, which would happen by memcpy if this function wasn't here.
}

void DeserializePositionAndVelocity(void *memory, unsigned byteLength, void *context, RakNet::BitStream *bitStream, unsigned int timePacketSent, PlayerID playerId)
{
	PositionAndVelocity *pav = (PositionAndVelocity*) memory;
	bitStream->ReadDelta(pav->position);
	bitStream->ReadDelta(pav->velocity);

	// Skip reading lastPosition, which would happen by memcpy if this function wasn't here.
}
bool ShouldSendUpdate(void *memory, void *lastSentValue, unsigned memoryByteLength, PlayerID destinationSystem,  unsigned int currentTime, unsigned int lastSendTime, void *context)
{
	PositionAndVelocity *pav = (PositionAndVelocity*) memory;
	PositionAndVelocity *lastPav = (PositionAndVelocity*) lastSentValue;

    return pav->velocity!=lastPav->velocity || pav->position!=lastPav->position;
}

bool InterpolateCallback(void *currentValue, void *targetValue, unsigned int currentTimeMS, unsigned int timeReceivedMS, unsigned int lastCallTime, void *context)
{
	PositionAndVelocity *current = (PositionAndVelocity*) currentValue;
	PositionAndVelocity *target = (PositionAndVelocity*) targetValue;

	return dataReplicator.InterpolateFloat(&(current->position), target->position, target->velocity, currentTimeMS, timeReceivedMS, lastCallTime, 2000, false);
}
class TestClass : public ReplicatedObject
{
public:
	bool inScope;
	// Game position and velocity
	PositionAndVelocity positionAndVelocity;

	TestClass::TestClass()
	{
		testClass=this;
		if (isServer)
			lastTime=lastDisplayTime=RakNet::GetTime();
		counter=0;
		positionAndVelocity.position=0.0f;
		positionAndVelocity.velocity=0.0f;
		positionAndVelocity.lastPosition=0.0f;

		dataReplicator.ReplicateObject("TestClass", this, ownObject, HIGH_PRIORITY, RELIABLE_ORDERED, 0, 1000);
	}
	TestClass::~TestClass()
	{
		testClass=0;
		dataReplicator.DereplicateObject(this, true);
	}

	void Update(unsigned time)
	{
		if (GetWaitingForCreationResponse())
			return;

		unsigned elapsed=time-lastTime;
		if (ownMemory)
			positionAndVelocity.position = positionAndVelocity.position + positionAndVelocity.velocity * elapsed;

		if (time-lastDisplayTime > 200.0f)
		{
			if (inScope)
				printf("[In ] ");
			else
				printf("[Out] ");	
			printf("%i: %f->%f vel=%f owner=%i:%i\n", counter, positionAndVelocity.lastPosition, positionAndVelocity.position, positionAndVelocity.velocity, GetOwner().binaryAddress, GetOwner().port);
			if (ownMemory)
				counter++;
			lastDisplayTime=time;
			positionAndVelocity.lastPosition=positionAndVelocity.position;
		}		
		lastTime=time;
	}

	// Interface implementation for ReplicatedObject
	bool InScope(PlayerID playerId, DataReplicator *dataReplicator)
	{
		return inScope;
	}

	// Interface implementation for ReplicatedObject
	void OnScopeChange(PlayerID sender, bool isNowInScope, DataReplicator *dataReplicator)
	{
		if (isNowInScope)
			printf("Object is now in scope from %i:%i\n", sender.binaryAddress, sender.port);
		else
			printf("Object is no longer in scope from %i:%i\n", sender.binaryAddress, sender.port);
		inScope=isNowInScope;
	}

	char* GetObjectClassName(void) const
	{
		return "TestClass";
	}

	virtual PlayerID GetNetworkIDAuthority(void) const
	{
		if (isServer)
			return rakServer->GetInternalID();
		else
			return rakClient->GetServerID();
	}
	virtual bool IsNetworkIDAuthority(void) const
	{
        return isServer;		
	}

	virtual ObjectReplicationPushResult OnSendReplicationPush(PlayerID destination, RakNet::BitStream *outContext, DataReplicator *dataReplicator)
	{
		return OBJECT_REPLICATION_OK;
	}
	virtual bool OnReceiveReplicationPush(PlayerID sender, RakNet::BitStream *inContext, ObjectID newObjectId, PlayerID &objectOwner, DataReplicator *dataReplicator)
	{
		return true;
	}

	void OnReplication(DataReplicator *dataReplicator)
	{
		puts("Own memory? (y/n)");
		if (getch()=='y')
		{
			ownMemory=true;

		}
		else
		{
			ownMemory=false;
		}

		// Passing false to isUnifiedMemory because I want unique values per system for lastSendValue for the serialize callback
		dataReplicator->SynchronizeMemory("positionAndVelocity", &positionAndVelocity, sizeof(PositionAndVelocity), ownMemory, 4000, this);
		// Change behavior of serialize and deserialize to exclude sending lastPosition
		dataReplicator->SetMemorySerializeCB(&positionAndVelocity, SerializePositionAndVelocity);
		dataReplicator->SetMemoryDeserializeCB(&positionAndVelocity, DeserializePositionAndVelocity);
		dataReplicator->SetMemoryShouldSendUpdateCB(&positionAndVelocity, ShouldSendUpdate);

		printf("Interpolate position and velocity (y/n)?\n");
		if (getch()=='y')
		{
			dataReplicator->InterpolateMemory(&positionAndVelocity, InterpolateCallback);
		}

		// You don't have to set callbacks for simple types, such as integers
		dataReplicator->SynchronizeMemory("counter", &counter, sizeof(counter), ownMemory, 500, this);

		testClass=this;
		objectSynchronized=true;
		printf("Object created and memory synchronized\n");
	}
	void OnRemoteDereplication(DataReplicator *dataReplicator)
	{
		printf("Object remotely deleted and unsynchronized\n");
		objectSynchronized=false;
		testClass=0;

		delete this;
	}

	bool OnReceiveCreationRequest(PlayerID sender, RakNet::BitStream *inContext, RakNet::BitStream *outContext, PlayerID &objectOwner, DataReplicator *dataReplicator)
	{
		puts("Got creation request.  Allow? (y/n)");
		if (getch()=='n')
			return false;

		char ch;
		puts("Override client and set new owner?");
		puts("1=Server owns object");
		puts("2=Client 0 owns object");
		puts("3(default)=Client owns object if requested, otherwise server owns object");
		ch=getch();

		if (ch=='1')
		{
			ownObject=true;
			puts("Server taking control of the object");
			objectOwner=rakServer->GetInternalID();
		}
		else if (ch=='2')
		{
			ownObject=false;
			puts("Setting owner to client 0.");
			objectOwner=rakServer->GetPlayerIDFromIndex(0);
		}
		else
		{
			if (objectOwner==UNASSIGNED_PLAYER_ID)
			{
				ownObject=true;
				puts("Server taking control of the object");
				objectOwner=rakServer->GetInternalID();
			}
			else
			{
				ownObject=false;
				puts("Client owns the object");
				puts("When this system disconnects, the object should be dereplicated");
			}
		}

		return true;
	}

	bool OnCreationRequestAcceptance(PlayerID sender, ObjectID newObjectId, RakNet::BitStream *inContext, PlayerID &objectOwner, DataReplicator *dataReplicator)
	{
		lastTime=lastDisplayTime=RakNet::GetTime();
		puts("Creation request accepted");
		return true;
	}
	void OnCreationRequestRejection(PlayerID sender, RakNet::BitStream *inContext, DataReplicator *dataReplicator)
	{
		puts("Creation request rejected");
		delete testClass;
		testClass=0;
	}

	unsigned counter;

protected:
	unsigned lastTime,lastDisplayTime;
};

void ShowHelp(void)
{
	printf("\n(I)nvert velocity.\n");
	printf("(C)reate test class.\n");
	printf("Change (S)cope of object.\n");
	printf("(D)ereplicate object.\n");
	printf("Reset (P)osition.\n");
	printf("(H)elp\n");
	printf("(Q)uit.\n\n");
}

ReplicatedObject* ObjectFactoryCallback(const char *className, RakNet::BitStream *context)
{
	if (strcmp(className, "TestClass")==0)
		return new TestClass;

	assert(0);
	return 0;
}

int main(void)
{
	rakClient=RakNetworkFactory::GetRakClientInterface();
	rakServer=RakNetworkFactory::GetRakServerInterface();
	Multiplayer<RakClientInterface> rakClientMultiplayer;
	Multiplayer<RakServerInterface> rakServerMultiplayer;

	dataReplicator.SetClassFactory(ObjectFactoryCallback);
	dataReplicator.SetBroadcastToAll(true);
	testClass=0;

	// Crude interface
	puts("Run as (1) Client or (2) Server ?");
	if (getch()=='1')
	{
		// Holds user data
		char ip[30], serverPort[30], clientPort[30];
		// A client
		isServer=false;

		// Get our input
		puts("Enter the client port to listen on");
		gets(clientPort);
		if (clientPort[0]==0)
			strcpy(clientPort, "0");
		puts("Enter IP to connect to");
		gets(ip);
		if (ip[0]==0)
			strcpy(ip, "127.0.0.1");
		puts("Enter the port to connect to");
		gets(serverPort);
		if (serverPort[0]==0)
			strcpy(serverPort, "60000");

		// Connecting the client is very simple.  0 means we don't care about
		// a connectionValidationInteger, and false for low priority threads
		bool b = rakClient->Connect(ip, atoi(serverPort), atoi(clientPort), 0, 30);

		if (b)
			puts("Attempting connection");
		else
		{
			puts("Bad connection attempt.  Terminating.");
			exit(1);
		}

		rakClient->AttachPlugin(&dataReplicator);
	}
	else
	{
		// Holds user data
		char portstring[30];


		// A server
		isServer=true;
		puts("Enter the server port to listen on");
		gets(portstring);
		if (portstring[0]==0)
			strcpy(portstring, "60000");

		puts("Starting server.");
		// Starting the server is very simple.  2 players allowed.
		// 0 means we don't care about a connectionValidationInteger, and false
		// for low priority threads
		bool b = rakServer->Start(10, 0, 30, atoi(portstring));
		if (b)
			puts("Server started, waiting for connections.");
		else
		{ 
			puts("Server failed to start.  Terminating.");
			exit(1);
		}

		rakServer->AttachPlugin(&dataReplicator);
	}

	ownObject=false;


	ShowHelp();
	
	char ch=0;

	while (1)
	{
		rakClientMultiplayer.ProcessPackets(rakClient);
		rakServerMultiplayer.ProcessPackets(rakServer);
		if (kbhit())
			ch=getch();

		if (ch=='h')
		{
			ShowHelp();
		}
		else if (ch=='c' || ch=='r')
		{
			if (testClass==0)
			{
				puts("Own object? (y/n)");
				if (getch()=='y')
				{
					ownObject=true;
					puts("This system owns the object.  When this system disconnects, the object should be dereplicated");
				}
				else
				{
					ownObject=false;
					puts("This system does not own the object.");
				}
				
				if (ch=='c')
					testClass = new TestClass;

				printf("Created Test Class\n");
			}
			else
				printf("Test class already exists\n");
		}
		else if (ch=='i' && testClass)
		{
			if (testClass->positionAndVelocity.velocity==0.0f)
				testClass->positionAndVelocity.velocity=.001f;
			else
				testClass->positionAndVelocity.velocity=.0f;
			printf("Change velocity\n");

		}
		else if (ch=='s' && testClass)
		{
			testClass->inScope=!testClass->inScope;
			printf("Changed scope\n");
		}
		else if (ch=='d' && testClass)
		{
			printf("Locally dereplicating object.\n");
			delete testClass;
			testClass=0;
		}
		else if (ch=='p' && testClass)
		{
			testClass->positionAndVelocity.position=0.0f;
			testClass->counter=0;
			printf("Locally resetting position and counter\n");
		}
		else if (ch=='n' && testClass)
		{
			if (testClass)
			{
				delete testClass;
				testClass=0;
			}
			else
				printf("Object is not currently synchronized.\n");
		}
		else if (ch=='q')
			break;

		ch=0;

		// Don't take all the CPU time.
#ifdef _WIN32
		Sleep(30);
#else
		usleep(30 * 1000);
#endif

		if (testClass)
			testClass->Update(RakNet::GetTime());
	}

	dataReplicator.Clear();

	RakNetworkFactory::DestroyRakClientInterface(rakClient);
	RakNetworkFactory::DestroyRakServerInterface(rakServer);

	return 0;
}
